################################
#### Code to generate FigS7 ####
################################

#### Load required packages
import numpy as np
from scipy import stats
import random
import cmocean
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib import rc
import matplotlib as mpl
from matplotlib.colorbar import Colorbar
from copy import deepcopy

cm_1 = cmocean.cm.amp
cm_2 = cmocean.cm.ice_r
cm_3 = cmocean.cm.tempo
cm_4 = cmocean.cm.rain

#### Load data
array_incu_infector = np.load("../data/array_incubation_infector.npy")
array_incu_infectee = np.load("../data/array_incubation_infectee.npy")
array_GI_raw = np.load("../data/array_GI_raw.npy")
array_SI_raw = np.load("../data/array_SI_raw.npy")

#### Down sample unlikely pairs based on incubation period distribution
array_p_incu_2d = []
for i in range(100):
    array_p_incu_2d.append([])
    for incu_infector, incu_infectee in zip(array_incu_infector[i], array_incu_infectee[i]):
        p_incu_infector = stats.weibull_min.pdf(incu_infector, 1.578104 , scale=7.112123)
        p_incu_infectee = stats.weibull_min.pdf(incu_infectee, 1.578104 , scale=7.112123)
        array_p_incu_2d[i].append(p_incu_infector * p_incu_infectee)
array_p_incu_2d = np.array(array_p_incu_2d)
array_weights = []
for i in range(100):
    incu_kernel_imp = stats.gaussian_kde([array_incu_infector[i], array_incu_infectee[i]])
    p_kernel_2d_imp = incu_kernel_imp([array_incu_infector[i], array_incu_infectee[i]])
    array_weights.append(array_p_incu_2d[i]/p_kernel_2d_imp)

array_GI = []
array_SI = []
for i in range(100):
    array_GI.append(random.choices(population=array_GI_raw[i], weights=array_weights[i], k=len(array_GI_raw[i])))
    array_SI.append(random.choices(population=array_SI_raw[i], weights=array_weights[i], k=len(array_SI_raw[i])))
array_GI = np.array(array_GI)
array_SI = np.array(array_SI)

list_GI = array_GI.flatten()
list_SI = array_SI.flatten()


#### Plot
cmaps = [cm_1, cm_2]
fig, axes = plt.subplots(2, 2, figsize = [8, 6])
for idx, temp_array in enumerate([array_GI, array_SI]):
    temp_list = temp_array.flatten()
    lower = np.percentile(temp_list, 25)
    median = np.median(temp_list)
    upper = np.percentile(temp_list, 75)

    lower_str = str(round(lower, 1))
    median_str = str(round(median, 1))
    upper_str = str(round(upper, 1))
    
    x_fit_CI = np.linspace(min(temp_list), max(temp_list), 10)
    x_fit = np.linspace(min(temp_list), max(temp_list), 200)

    array_y_fit_cdf = []
    for row in temp_array:
        row_sorted = np.sort(row)
        cdf = 1. * np.arange(len(row_sorted)) / (len(row_sorted) - 1)
        if idx == 0:
            shape, loc, scale = stats.weibull_min.fit(row, floc=0)
            array_y_fit_cdf.append(stats.weibull_min.cdf(x_fit_CI, shape, loc, scale=scale))
        else:
            shape, loc, scale = stats.lognorm.fit(row)
            array_y_fit_cdf.append(stats.lognorm.cdf(x_fit_CI, shape, loc, scale=scale))
        axes[0][idx].plot(row_sorted, cdf, linewidth=0.5, color=cm_2(0.3))
    array_y_fit_cdf = np.array(array_y_fit_cdf)  

    if idx == 0:
        #### fit gamma distribution ####
        shape, loc, scale = stats.gamma.fit(temp_list, floc=0)
        y_fit_gamma = stats.gamma.pdf(x_fit, shape, loc, scale=scale)
        y_fit_gamma_cdf = stats.gamma.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.gamma.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_gamma_cdf,
            linestyle = '--',
            linewidth=1.,
            color=cm_2(0.7),
            label="gamma: AIC="+str(AIC))

        #### fit weibull distribution ####
        shape, loc, scale = stats.weibull_min.fit(temp_list, floc=0)
        y_fit_weibull_pdf = stats.weibull_min.pdf(x_fit, shape, loc, scale=scale)
        y_fit_weibull_cdf = stats.weibull_min.cdf(x_fit, shape, loc, scale=scale)
        y_fit_weibull_cdf_CI = stats.weibull_min.cdf(x_fit_CI, shape, loc, scale=scale)
        AIC = 2 - int(stats.weibull_min.logpdf(temp_list, shape, loc, scale).sum())
        params_str = "shape: "+str(round(shape, 2))+", "+"scale: "+str(round(scale, 2))
        axes[1][idx].plot(
            x_fit, y_fit_weibull_pdf,
            linestyle='-',
            color=cm_1(0.7),
            linewidth=2,
            label="best fit")
        axes[0][idx].plot(
            x_fit, y_fit_weibull_cdf,
            linestyle='-',
            color=cm_1(0.7),
            linewidth=2,
            label="weibull: AIC="+str(AIC))
        y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
        y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
        y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)
        axes[0][idx].vlines(x_fit_CI, y_fit_cdf_lower, y_fit_cdf_upper, zorder=100, linewidth=1, color=cm_1(0.7))
        
        #### fit lognormal distribution ####
        shape, loc, scale = stats.lognorm.fit(temp_list, floc=0)
        y_fit_lognormal = stats.lognorm.pdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf = stats.lognorm.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.lognorm.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_lognormal_cdf,
            linestyle=":",
            linewidth=1.,
            color=cm_3(0.7),
            label="lognormal: AIC="+str(AIC))
        
    else:
        #### fit gamma distribution ####
        shape, loc, scale = stats.gamma.fit(temp_list)
        y_fit_gamma = stats.gamma.pdf(x_fit, shape, loc, scale=scale)
        y_fit_gamma_cdf = stats.gamma.cdf(x_fit, shape, loc, scale=scale)
        AIC = 3 - int(stats.gamma.logpdf(temp_list, shape, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_gamma_cdf,
            linestyle=":",
            linewidth=1.,
            color=cm_3(0.8),
            label="gamma: AIC="+str(AIC))
        
        #### fit normal distribution ####
        loc, scale = stats.norm.fit(temp_list)
        y_fit_norm = stats.norm.pdf(x_fit, loc, scale=scale)
        y_fit_norm_cdf = stats.norm.cdf(x_fit, loc, scale=scale)
        AIC = 2 - int(stats.norm.logpdf(temp_list, loc, scale).sum())
        axes[0][idx].plot(
            x_fit, y_fit_norm_cdf,
            linestyle="--",
            color=cm_2(0.7),
            linewidth=1.,
            label="normal: AIC="+str(AIC))
        
        #### fit lognormal distribution ####
        shape, loc, scale = stats.lognorm.fit(temp_list)
        y_fit_lognormal = stats.lognorm.pdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf = stats.lognorm.cdf(x_fit, shape, loc, scale=scale)
        y_fit_lognormal_cdf_CI = stats.lognorm.cdf(x_fit_CI, shape, loc, scale=scale)
        AIC = 3 - int(stats.lognorm.logpdf(temp_list, shape, loc, scale).sum())
        axes[1][idx].plot(
            x_fit, y_fit_lognormal,
            linestyle="-",
            linewidth=2,
            color=cm_1(0.7),
            label="best fit")
        axes[0][idx].plot(
            x_fit, y_fit_lognormal_cdf,
            linestyle="-",
            linewidth=2,
            color=cm_1(0.7),
            label="lognormal: AIC="+str(AIC))
        y_fit_cdf_lower = np.percentile(array_y_fit_cdf, 2.5, axis=0)
        y_fit_cdf_upper = np.percentile(array_y_fit_cdf, 97.5, axis=0)
        y_fit_cdf_avg = np.average(array_y_fit_cdf, axis=0)
        axes[0][idx].vlines(x_fit_CI, y_fit_cdf_lower, y_fit_cdf_upper, zorder=100, linewidth=1, color=cm_1(0.7))

    bar_width = (max(temp_list) - min(temp_list))/16
    axes[1][idx].hist(
        temp_list,
        bins=15,
        color="0.8",
        density=True,
        width=bar_width,
        label="data")

    if idx == 0:
        label_str = median_str + " ("  +lower_str + ", " + upper_str + ")"
        axes[0][idx].set_title("(A)", fontsize=10, loc="left")
        axes[1][idx].set_title("(C)", fontsize=10, loc="left")
        axes[0][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="lower right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[1][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="upper right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[0][idx].set_xlabel("generation interval (days)", fontsize=8)
        axes[1][idx].set_xlabel("generation interval (days)", fontsize=8)
    else:
        axes[0][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="lower right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        axes[1][idx].legend(fontsize=6, frameon=False, framealpha=None, loc="upper right", edgecolor="0.8", fancybox=False, borderaxespad=1)
        label_str = median_str + " ("  +lower_str + ", " + upper_str + ")" 
        axes[0][idx].set_title("(B)", fontsize=10, loc="left")
        axes[1][idx].set_title("(D)", fontsize=10, loc="left")
        axes[0][idx].set_xlabel("serial interval (days)", fontsize=8)
        axes[1][idx].set_xlabel("serial interval (days)", fontsize=8)

    axes[1][idx].set_ylabel("Probability density function", fontsize=8)
    axes[0][idx].set_ylabel("Cumulative distribution function", fontsize=8)
    axes[0][idx].tick_params(axis="both", labelsize=8)
    axes[1][idx].tick_params(axis="both", labelsize=8)  
    if idx == 0:
        axes[1][idx].set_ylim(-0.01, 0.16)
    else:
        axes[1][idx].set_ylim(-0.01, 0.12)
    
plt.tight_layout()
plt.savefig("../plots/FigS7.pdf")